@@ -135,16 +135,25 @@ def IFN(S, A, B):
     return np.where(not S, A, B)
 
 
-def formula_apply(S, N, func: Callable[..., Any], raw: bool = False):
+def rolling_apply(S, N, func: Callable[..., Any], raw: bool = False):
+    """
+    Rolling(N).apply()的序列化版本
+    :param S:
+    :param N:
+    :param func:
+    :param raw:
+    :return:
+    """
     s_len = len(S)
-    res = np.repeat(np.nan, s_len)
+    ret = np.repeat(np.nan, s_len)
+    if isinstance(N, (int, float)):
+        N = np.repeat(N, s_len)
     for i in range(s_len):
         if (not np.isnan(N[i])) and N[i] <= i + 1:
             window = N[i]
-            t = S[i + 1 - window:i + 1]
-            # res[i] = np.polyval(np.polyfit(range(window), t, deg=1), window - 1)
-            res[i] = func(t, window)
-    return res
+            T = S[i + 1 - window:i + 1]
+            ret[i] = func(T, window)
+    return ret
 
 
 def REF(S, N=1):
@@ -269,6 +278,15 @@ def LLVBARS(S, N):
 
 
 def MA(S, N):
+    """
+    求序列的N日简单移动平均值，返回序列
+    :param S:
+    :param N:
+    :return:
+    """
+    return rolling_apply(S, N, lambda T, W: T.mean())
+
+def V1_MA(S, N):
     """
     求序列的N日简单移动平均值，返回序列
     :param S:
@@ -332,10 +350,10 @@ def AVEDEV(S, N):
     :param N:
     :return:
     """
-    return formula_apply(S, N, lambda R, W: (np.abs(R - R.mean())).mean(), raw=True)
+    return rolling_apply(S, N, lambda R, W: (np.abs(R - R.mean())).mean(), raw=True)
 
 
-def AVEDEV_old(S, N):
+def V1_AVEDEV(S, N):
     """
     平均绝对偏差, (序列与其平均值的绝对差的平均值)
     :param S:
@@ -376,11 +394,11 @@ def FORCAST(S, N):
     :param N: N要支持序列
     :return:
     """
-    return formula_apply(S, N, lambda R, W: np.polyval(np.polyfit(range(W), R, deg=1), W - 1) if W > 1 else np.nan,
+    return rolling_apply(S, N, lambda R, W: np.polyval(np.polyfit(range(W), R, deg=1), W - 1) if W > 1 else np.nan,
                          raw=True)
 
 
-def FORCAST_v2(S, N):
+def V2_FORCAST(S, N):
     """
     返回S序列N周期回线性回归后的预测值, jqz1226改进成序列出
     :param S: 目标序列
@@ -401,7 +419,7 @@ def FORCAST_v2(S, N):
         return res
 
 
-def FORCAST_v1(S, N):
+def V1_FORCAST(S, N):
     """
     返回S序列N周期回线性回归后的预测值, jqz1226改进成序列出
     :param S: 目标序列
@@ -420,8 +438,9 @@ def FORCAST_v1(S, N):
                 window = N[i]
                 pos = i + 1
                 t = S[pos - window:pos]
-                d = pd.Series(t).rolling(window).apply(
-                    lambda x: np.polyval(np.polyfit(range(window), x, deg=1), window - 1),
+                R = window
+                d = pd.Series(t).rolling(R).apply(
+                    lambda x: np.polyval(np.polyfit(range(R), x, deg=1), window - 1),
                     raw=True).values
                 res[pos - 1:pos] = d[window - 1:window]
             start += 1
@@ -510,10 +529,23 @@ def BARSSINCEN(S, N):
     return pd.Series(S).rolling(N).apply(lambda x: N - 1 - np.argmax(x) if np.argmax(x) or x[0] else 0,
                                          raw=True).fillna(0).values.astype(int)
 
-
 def CROSS(S1, S2):
     """
-    判断向上金叉穿越 CROSS(MA(C,5),MA(C,10))  判断向下死叉穿越 CROSS(MA(C,10),MA(C,5))
+    判断向上金叉穿越 CROSS(MA(C,5),MA(C,10))
+    判断向下死叉穿越 CROSS(MA(C,10),MA(C,5))
+    :param S1:
+    :param S2:
+    :return:
+    """
+    #return np.concatenate(([False], np.logical_not((S1 > S2)[:-1]) & (S1 > S2)[1:]))  # 不使用0级函数,移植方便  by jqz1226
+    c1 = REF(S1,1) < REF(S2,1)
+    c2 = S1 >= S2
+    return (c1 & c2)
+
+def V1_CROSS(S1, S2):
+    """
+    判断向上金叉穿越 CROSS(MA(C,5),MA(C,10))
+    判断向下死叉穿越 CROSS(MA(C,10),MA(C,5))
     :param S1:
     :param S2:
     :return:
