@@ -11,7 +11,7 @@ def f89k(data:pandas.DataFrame):
     :param code:
     :return:
     """
-    df = data
+    df = data.copy()
     OPEN = df['open']
     CLOSE = df['close']
     HIGH = df['high']
