@@ -8,7 +8,7 @@ def hjk(code:str, data:pandas.DataFrame):
     """
     黄金坑
     """
-    df = data
+    df = data.copy()
     OPEN = df['open']
     CLOSE = df['close']
     HIGH = df['high']
